//+------------------------------------------------------------------+
//|                                                     PMsimple.mq4 |
//|                                                            keiji |
//|                                     http://samuraifx.seesaa.net/ |
//+------------------------------------------------------------------+
#property copyright "keiji"
#property link      "http://samuraifx.seesaa.net/"


// Regular variables

extern double  MaxLotSize                    = 5;
extern double  LotSize                       = 0.1;
extern double  LotIncrement                  = 0.1;
extern double  ProfitTarget                  = 5;
extern double  TrendProfitTarget             = 5;
extern int     Spacing                       = 10;
extern int     TrendSpacing                  = 10;
extern int     TrendTimeFrame                = PERIOD_M1;
extern int     TrendPeriods                  = 1200;

// Internal settings
int            Order          = 0;
int            Orders         = 0;
int            Slippage       = 0;
string         TradeComment   = "PM";
datetime       BarTime        = 0;
static bool    TradeAllowed   = true;
double         TickPrice      = 0;
double         BuyProfits [1][2];
double         SellProfits [1][2];      


void CloseBuysInProfit()
{
   double BuyProfit, LastBuyTime;

   RefreshRates();

   Orders = ArraySize(BuyProfits) / 2;
   for (Order = 0; Order < Orders; Order++)
   {
      if (BuyProfits[Order][0] > 0 )
      {
         if (OrderSelect(BuyProfits[Order][1], SELECT_BY_TICKET))
         {
            if (OrderOpenTime() > LastBuyTime) LastBuyTime = OrderOpenTime();
            OrderClose(OrderTicket(), OrderLots(), Bid, 5, Green);
         }
      }
   }
   return;
}


void CloseSellsInProfit()
{
   double SellProfit, LastSellTime;

   RefreshRates();
   
   Orders = ArraySize(SellProfits) / 2;
   for (Order = 0; Order < Orders; Order++)
   {
      if (SellProfits[Order][0] > 0 )
      {
         if (OrderSelect(SellProfits[Order][1], SELECT_BY_TICKET))
         {
            if (OrderOpenTime() > LastSellTime) LastSellTime = OrderOpenTime();
            OrderClose(OrderTicket(), OrderLots(), Ask, 5, Red);
         }
      }
   }

   return;
}


void PlaceBuyOrder()
{
   double BuyOrders, Lots;
   double LowestBuy = 1000, HighestBuy;

   if (BarTime != Time[0])
   {
      BarTime = Time[0];
      TickPrice = 0;
      TradeAllowed = true;
   }

   RefreshRates();

   
   for (Order = OrdersTotal() - 1; Order >= 0; Order--)
   {
      if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == Symbol() && OrderType() == OP_BUY)
         {
            if (OrderOpenPrice() < LowestBuy) LowestBuy = OrderOpenPrice();
            if (OrderOpenPrice() > HighestBuy) HighestBuy = OrderOpenPrice();
            BuyOrders++;
         }
      }
   }

   if (TradeAllowed)
   {
      if (Ask > HighestBuy + (TrendSpacing * Point))
      {
            Lots = NormalizeDouble(LotSize + (LotIncrement * BuyOrders), 1);
      }

      if (Ask < LowestBuy - (Spacing * Point))
      {
            Lots = NormalizeDouble((LotSize * 2) + (LotIncrement * BuyOrders), 1);
      }

      if (Lots == 0) Lots = NormalizeDouble(LotSize, 1);
      
      if (Lots > MaxLotSize) Lots = MaxLotSize;

      if(OrderSend(Symbol(), OP_BUY, Lots, Ask, Slippage, 0, 0, TradeComment, 0, Green)>0)
      {
         TickPrice = Close[0];
         TradeAllowed = false;
      }
   }
}


void PlaceSellOrder()
{
   double SellOrders, Lots;
   double HighestSell, LowestSell = 1000;

   if (BarTime != Time[0])
   {
      BarTime = Time[0];
      TickPrice = 0;
      TradeAllowed = true;
   }

   RefreshRates();

   for (Order = OrdersTotal() - 1; Order >= 0; Order--)
   {
      if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == Symbol() &&  OrderType() == OP_SELL)
         {
            if (OrderOpenPrice() > HighestSell) HighestSell = OrderOpenPrice();
            if (OrderOpenPrice() < LowestSell) LowestSell = OrderOpenPrice();
            SellOrders++;
         }
      }
   }

   if (TradeAllowed)
   {
      if (Bid < LowestSell - (TrendSpacing * Point))
      {
            Lots = NormalizeDouble(LotSize + (LotIncrement * SellOrders), 1);
      }
      
      if (Bid > HighestSell + (Spacing * Point))
      {
            Lots = NormalizeDouble((LotSize * 2) + (LotIncrement * SellOrders), 1);
      }

      if (Lots == 0) Lots = NormalizeDouble(LotSize, 1);
      
      if (Lots > MaxLotSize) Lots = MaxLotSize;

      if(OrderSend(Symbol(), OP_SELL, Lots, Bid, Slippage, 0, 0, TradeComment, 0, Red)>0)
      {
         TickPrice = Close[0];
         TradeAllowed = false;
      }
   }
}


int start()
{
   double         MarginPercent;
   static double  LowMarginPercent = 10000000, LowEquity = 10000000, LowMargin = 10000000;
   double         BuyPipTarget, SellPipTarget;
   double         BuyPips, SellPips, BuyLots, SellLots;
   int            BuyOrders = 0, SellOrders = 0;
   double         LowestBuy = 999, HighestBuy = 0.0001, LowestSell = 999, HighestSell = 0.0001, HighPoint, MidPoint, LowPoint;
   double         Profit, BuyProfit, SellProfit, PosBuyProfit, PosSellProfit;
   int            HighestBuyTicket, LowestBuyTicket, HighestSellTicket, LowestSellTicket;
   double         HighestBuyProfit, LowestBuyProfit, HighestSellProfit, LowestSellProfit;
   double         CurrentTime = (TimeHour(CurTime() + TimeMinute(CurTime())));
   double         Trend, TrendPrev;
   bool           SELLme = false;
   bool           BUYme = false;
   double         Margin = MarketInfo(Symbol(), MODE_MARGINREQUIRED);
   string         Message;
   
   
   

   Orders = OrdersTotal();
   ArrayResize(BuyProfits, Orders + 1);
   ArrayResize(SellProfits, Orders + 1);
   ArrayInitialize(BuyProfits, 0);
   ArrayInitialize(SellProfits, 0);

   for (Order = Orders - 1; Order >= 0; Order--)
   {
      if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == Symbol() )
         {
            Profit = OrderProfit() + OrderSwap() + OrderCommission();
            
            if (OrderType() == OP_BUY)
            {
               if (OrderOpenPrice() > HighestBuy)
               {
                  HighestBuy = OrderOpenPrice();
                  HighestBuyTicket = OrderTicket();
                  HighestBuyProfit = Profit;
               }

               if (OrderOpenPrice() < LowestBuy)
               {
                  LowestBuy = OrderOpenPrice();
                  LowestBuyTicket = OrderTicket();
                  LowestBuyProfit = Profit;
               }

               BuyProfits[BuyOrders][0] = Profit;
               BuyProfits[BuyOrders][1] = OrderTicket();

               BuyOrders++;
               BuyLots += OrderLots();

               BuyProfit += Profit;
            }

            if (OrderType() == OP_SELL)
            {
               if (OrderOpenPrice() > HighestSell)
               {
                  HighestSell = OrderOpenPrice();
                  HighestSellTicket = OrderTicket();
                  HighestSellProfit = Profit;
               }

               if (OrderOpenPrice() < LowestSell)
               {
                  LowestSell = OrderOpenPrice();
                  LowestSellTicket = OrderTicket();
                  LowestSellProfit = Profit;
               }

               SellProfits[SellOrders][0] = Profit;
               SellProfits[SellOrders][1] = OrderTicket();

               SellOrders++;
               SellLots += OrderLots();

               SellProfit += Profit;
            }
         }
      }
   }

   ArraySort(BuyProfits, WHOLE_ARRAY, 0, MODE_DESCEND);
   ArraySort(SellProfits, WHOLE_ARRAY, 0, MODE_DESCEND);
      
   Orders = ArraySize(BuyProfits) / 2;

   for (Order = 0; Order < Orders; Order++)
   {
      if (BuyProfits[Order][0] > 0) PosBuyProfit += BuyProfits[Order][0];
      if (SellProfits[Order][0] > 0) PosSellProfit += SellProfits[Order][0];
   }


      BuyPipTarget = ProfitTarget;
      SellPipTarget = ProfitTarget;


   HighPoint = MathMax(HighestBuy, HighestSell);
   LowPoint = MathMin(LowestBuy, LowestSell);
   MidPoint = (HighPoint + LowPoint) / 2;

   RefreshRates();

   if (BuyOrders + SellOrders > 1)
   {
      if (Ask > MidPoint)
      {
         if (PosBuyProfit + LowestSellProfit >= ProfitTarget && LowestSell < LowestBuy)
         {
            OrderSelect(LowestSellTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Ask, 5, Green))
            {
               CloseBuysInProfit();

               LowestBuy = 1000;
               HighestBuy = 0;
               LowestSell = 1000;
               LowestSellTicket = 0;
            }
         }
         else if (PosSellProfit + LowestSellProfit >= ProfitTarget && LowestSell < LowestBuy)
         {
            OrderSelect(LowestSellTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Ask, 5, Red))
            {
               CloseSellsInProfit();

               LowestSell = 1000;
               HighestSell = 0;
               LowestSellTicket = 0;
            }
         }
         else if (PosBuyProfit + LowestBuyProfit >= ProfitTarget && LowestBuy < LowestSell)
         {
            OrderSelect(LowestBuyTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Bid, 5, Green))
            {
               CloseBuysInProfit();

               LowestBuy = 1000;
               HighestBuy = 0;
               LowestBuyTicket = 0;
            }
         }
         else if (PosSellProfit + LowestBuyProfit >= ProfitTarget && LowestBuy < LowestSell)
         {
            OrderSelect(LowestBuyTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Bid, 5, Red))
            {
               CloseSellsInProfit();

               LowestSell = 1000;
               HighestSell = 0;
               LowestBuy = 1000;
               LowestBuyTicket = 0;
            }
         }
      }
      else if (Bid < MidPoint)
      {
         if (PosBuyProfit + HighestBuyProfit >= ProfitTarget && HighestBuy > HighestSell)
         {
            OrderSelect(HighestBuyTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Bid, 5, Green))
            {
               CloseBuysInProfit();

               LowestBuy = 1000;
               HighestBuy = 0;
               HighestBuyTicket = 0;
            }
         }
         else if (PosSellProfit + HighestBuyProfit >= ProfitTarget && HighestBuy > HighestSell)
         {
            OrderSelect(HighestBuyTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Bid, 5, Red))
            {
               CloseSellsInProfit();

               LowestSell = 1000;
               HighestSell = 0;
               HighestBuy = 0;
               HighestBuyTicket = 0;
            }
         }
         else if (PosBuyProfit + HighestSellProfit >= ProfitTarget && HighestSell > HighestBuy)
         {
            OrderSelect(HighestSellTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Ask, 5, Green))
            {
               CloseBuysInProfit();

               LowestBuy = 1000;
               HighestBuy = 0;
               HighestSell = 0;
               HighestSellTicket = 0;
            }
         }
         else if (PosSellProfit + HighestSellProfit >= ProfitTarget && HighestSell > HighestBuy)
         {
            OrderSelect(HighestSellTicket, SELECT_BY_TICKET);

            if(OrderClose(OrderTicket(), OrderLots(), Ask, 5, Red))
            {
               CloseSellsInProfit();

               LowestSell = 1000;
               HighestSell = 0;
               HighestSellTicket = 0;
            }
         }
      }
   }

   RefreshRates();

   Trend = iMA(Symbol(), TrendTimeFrame, TrendPeriods, 0, MODE_LWMA, PRICE_CLOSE, 0);
   TrendPrev = iMA(Symbol(), TrendTimeFrame, TrendPeriods, 0, MODE_LWMA, PRICE_CLOSE, 1);
   
if (Trend == 0) return;
   // BUY Trade Criteria
   if (HighestBuy > 0 && LowestBuy < 1000)
   {
      if (Ask < LowestBuy - (Spacing * Point) || Ask > HighestBuy + (TrendSpacing * Point))
      {
         BUYme = true;
         if (TickPrice > 0 && Close[0] < TickPrice) TradeAllowed = true;
      }

      if (TrendPrev > Trend) BUYme = false;
      if (BUYme) PlaceBuyOrder();
   }

   // SELL Trade Criteria
   if (HighestSell > 0 && LowestSell < 1000)
   {
      if (Bid > HighestSell + (Spacing * Point) || Bid < LowestSell - (TrendSpacing * Point))
      {
         SELLme = true;
         if (TickPrice > 0 && Close[0] > TickPrice) TradeAllowed = true;
      }
      
      if (TrendPrev < Trend) SELLme = false;
      if (SELLme) PlaceSellOrder();
   }

   if (ObjectFind("TrendAxis") != 0)
   {
      ObjectCreate("TrendAxis", OBJ_HLINE, 0, Time[0], Trend);
      ObjectSet("TrendAxis", OBJPROP_COLOR, Red);
      ObjectSet("TrendAxis", OBJPROP_WIDTH, 2);
   }
   else
   {
      ObjectMove("TrendAxis", 0, Time[0], Trend);
   }
}


